"""
This module provides main imports.
"""

from app.api.v1.roles import router as roles_router
from app.api.v1.users import router as users_router

__all__ = [
    "roles_router",
    "users_router",
]
