# FASTAPI Project Kit
"This is a FastAPI project generated by fastapi-boilerplate. Follow the given steps to start the server."

# Change directory
cd {{project_name}}

# Install dependencies
pip install -r requirements.txt

# Start the Server
py main.py

# Swagger Docs URL
http://localhost:8000/docs

# Project name
{{project_name}}
